@@ -145,6 +145,9 @@ static char rcsid[] = "$Id: uurate.c,v 1.15 1994/04/07 21:47:11 kdburg Rel $";
 #include "uucp.h"
 /* uucp.h includes string.h or strings.h, no include here. */
 
+#if HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
 
 #define _DEBUG_ 0
 
